namespace utils{
	template<typename T>
	class com_ptr{
		T* ptr;
	public:
		typedef T value_type;
		com_ptr(const com_ptr<T>&src){
			src->AddRef();
			ptr=src.ptr;
		}
		com_ptr(com_ptr<T>&&src):ptr(src.ptr){
			src.ptr = nullptr;
		}
		com_ptr():ptr(nullptr){}
		explicit com_ptr(T *ptr_):ptr(ptr_){}
		template<typename U>
		explicit com_ptr(const com_ptr<U>& src):ptr(nullptr){
			src.QueryInterface(*this);
		}
		~com_ptr(){if(ptr)ptr->Release();}
	
		void swap(com_ptr<T>& other)throw(){
			std::swap(ptr,other.ptr);
		}
	
		com_ptr<T>& operator=(com_ptr<T> src){
			this->swap(src);
			return *this;
		}
		bool operator!()const{return !ptr;}
		operator T*()const{return ptr;}
		T* operator->()const{return ptr;}
		T& operator*()const{return *ptr;}
		T** operator&(){Release();return &ptr;}
		bool operator==(const com_ptr<T>& right)const{return ptr==right.ptr;}
		bool operator!=(const com_ptr<T>& right)const{return !(*this==right);}
		
		template<typename U>
		HRESULT QueryInterface(com_ptr<U>&refPtr)const{
			return QueryInterface(&refPtr);
		}
		template<typename U>
		HRESULT QueryInterface(U** pptr)const{
			return QueryInterface(IID_PPV_ARGS(pptr));
		}
		HRESULT QueryInterface(REFIID riid,void** ppvObject)const{
			return ptr->QueryInterface(riid,ppvObject);
		}
		ULONG AddRef(){
			return ptr->AddRef();
		}
		ULONG Release(){
			if(ptr)ptr->Release();
			ptr = nullptr;
		}
	};

	template<typename T>
	inline com_ptr<T> make_comptr(T* ptr){
		return com_ptr<T>(ptr);
	}
}

MIDL_INTERFACE(/*IID*/) IComSample : IUnknown{
	virtual HRESULT STDMETHODCALLTYPE ShowMessage(LPCWSTR pszMessage) = 0;
};

// ϏIUnknownC^[tFCXB
interface INonDelegatingUnknown{
	virtual HRESULT STDMETHODCALLTYPE QueryInterfaceNonDelegate(REFIID riid,void** ppvObject) = 0;
	virtual ULONG STDMETHODCALLTYPE AddRefNonDelegate() = 0;
	virtual ULONG STDMETHODCALLTYPE ReleaseNonDelegate() = 0;
};

class CInnerSample : public IComSample, public INonDelegatingUnknown{
	CInnerSample();
	~CInnerSample(){}
	
public:
	static HRESULT Create(LPUNKNOWN pUnkOuter,REFIID riid,void** ppvObject);
	
	// AE^[IuWFNg֊ۓIUnknowni2ڂ̗vɑΉjB
	STDMETHODIMP QueryInterface(REFIID riid,void** ppvObject){
		return pUnkOuter->QueryInterface(riid,ppvObject);
	}
	STDMETHODIMP_(ULONG) AddRef(){
		return pUnkOuter->AddRef();
	}
	STDMETHODIMP_(ULONG) Release(){
		return pUnkOuter->Release();
	}
	
	// IComSample̎
	STDMETHODIMP ShowMessage(LPCWSTR pszMessage){
		MessageBoxW(nullptr,pszMessage,L"Inner Sample!",MB_OK);
		return S_OK;
	}
	
	// Ci[IuWFNggIUnknowni1ڂ̗vɑΉjB
	STDMETHODIMP QueryInterfaceNonDelegate(REFIID riid,void** ppvObject){
		if(!ppvObject) return E_INVALIDARG;
		*ppvObject = nullptr;
		
		if(riid == IID_IUnknown){
			// IUnknownvꂽƂ́AϏIUnknowniȂ킿Ci[IuWFNggIUnknownjԂB
			*ppvObject = (INonDelegatingUnknown*)this; //LXgYƂƂłȂڂɑB
		}else if(riid == IID_IComSample){
			// IUnknownȊOvꂽƂ́AϏIUnknownԂi2ڂ̗vɑΉjB
			*ppvObject = (IComSample*)this;
		}else{
			return E_NOINTERFACE;
		}
		return S_OK;
	}
	STDMETHODIMP_(ULONG) AddRefNonDelegate(){
		return ++refCount;
	}
	STDMETHODIMP_(ULONG) ReleaseNonDelegate(){
		if(--refCount == 0){
			delete this;
			return 0;
		}
		return refCount;
	}
private:
	IUnknown *pUnkOuter;
	ULONG refCount;
};

/*
IuWFNg̃R[h
*/
CInnerSample::CInnerSample(LPUNKNOWN pUnkOuter_):refCount(1){
	// AggregationȂꍇAthisAE^[IuWFNg̃|C^ƂĐݒ肵ĂB
	if(!pUnkOuter_)pUnkOuter_ = (INonDelegatingUnknown*)this; //LXgYƂƂłȂڂɑB
	
	// ϏIuWFNg̃AE^[IuWFNg̐ݒB
	pUnkOuter = pUnkOuter_;
}

HRESULT CInnerSample::Create(LPUNKNOWN pUnkOuter,REFIID riid,void** ppvObject){
	if(!ppvObject) return E_INVALIDARG;
	*ppvObject = nullptr;
	
	// FpUnkOuterNULLłȂAvC^[tFCXIUnknownłȂꍇAsi5ڂ̗vɑΉjB
	if((!pUnkOuter)&&(riid != IID_IUnknown))return E_NOINTERFACE;
	
	utils::com_ptr<CInnerSample> pObject;
	try{
		pObject = utils::make_comptr(new CInnerSample(pUnkOuter));
	}catch(std::bad_alloc&){
		return E_OUTOFMEMORY;
	}
	return pObject.QueryInterface(riid,ppvObject);
}
